/********************************************/
/*    電池放電テスト                        */
/********************************************/
//    外部A/D AD7890AN-10 12bit 8ch ±10V入力
//    外部D/A LTC1451     12bit
//    2013-09-03    開始
//         10-21    現行
//
//    PB0    o  A/D CS (TFS,RFS)
//    PB1    o  A/D CLKIN (OCA1A)
//    PB2    i  A/D DOUT
//    PB3    o  A/D SCLK
//    PB4    o  A/D DIN
//    PC0    o  D/A CLK
//    PC1    o  D/A DIN
//    PC2    o  D/A CS
//    PC3    i  CC ｴﾗｰ (Lでｴﾗｰ)
//    PC4    o  CC on (Hでｵﾝ)
//    PC5    i  SW (Lでｵﾝ)
//    ﾀｲﾏｰ1  A/D用ｸﾛｯｸ出力
//    ﾀｲﾏｰ2  1mSｼｽﾃﾑﾀｲﾏｰ

/*****  ライブラリ      ****/
#include  <MsTimer2.h>
#include  "wiring_private.h"      // sbi,cbi命令用

/*****  設定値      *****/
#define  ADSCL_V    975      // A/Dｽｹｰﾘﾝｸﾞ電圧値 x.xxV
#define  ADSCL_B    2000     // A/Dｽｹｰﾘﾝｸﾞデータ
#define  DASCL_A    1041     // D/Aｽｹｰﾘﾝｸﾞ電流値 xx.xxA
#define  DASCL_B    4000     // D/Aｽｹｰﾘﾝｸﾞデータ

/*****  ピン制御マクロ    *****/
#define  ADCS_H    (sbi(PORTB,PORTB0))
#define  ADCS_L    (cbi(PORTB,PORTB0))
#define  ADSCLK_H  (sbi(PORTB,PORTB3))
#define  ADSCLK_L  (cbi(PORTB,PORTB3))
#define  ADDIN_H   (sbi(PORTB,PORTB4))
#define  ADDIN_L   (cbi(PORTB,PORTB4))
#define  DACLK_H   (sbi(PORTC,PORTC0))
#define  DACLK_L   (cbi(PORTC,PORTC0))
#define  DADIN_H   (sbi(PORTC,PORTC1))
#define  DADIN_L   (cbi(PORTC,PORTC1))
#define  DACS_H    (sbi(PORTC,PORTC2))
#define  DACS_L    (cbi(PORTC,PORTC2))
#define  CC_ON     (sbi(PORTC,PORTC4))
#define  CC_OFF    (cbi(PORTC,PORTC4))
#define  ADDO_INP  (PINB & (1 << PORTB2))
#define  CCER_INP  (PINC & (1 << PORTC3))
#define  SW_INP    (PINC & (1 << PORTC5))

/******************************/
/*      タイマー割り込み      */
/******************************/

/*****  タイマー値    *****/
volatile boolean f_tm1ms;    // 1msありﾌﾗｸﾞ 
volatile byte  tm_1ms;       // 1msﾀﾞｳﾝｶｳﾝﾄﾀｲﾏｰ max 255ms
volatile word  tw_1ms;       // 1msﾀﾞｳﾝｶｳﾝﾄﾀｲﾏｰ max 65535ms
//    計測時間
volatile word  mtm_ms;       // 1ms upｶｳﾝﾄﾀｲﾏｰ 0～999
volatile byte  mtm_sec;      // 1秒 upｶｳﾝﾄﾀｲﾏｰ 0～59
volatile word  mtm_min;      // 1分 upｶｳﾝﾄﾀｲﾏｰ 0～9999
volatile boolean f_mtm1m;    // 1分ありﾌﾗｸﾞ 

/*****  タイマー2 1mS割り込み    *****/
//      wordのﾀｲﾏｰはﾒｲﾝで計時
void tm2int(void)
{
    sbi(PIND,PIND6);          // 12pin ﾄｸﾞﾙ
//  1ms計時処理
    f_tm1ms = 1;
    if(tm_1ms)    tm_1ms--;
}

/*****  計測時間ﾀｲﾏｰ計時    *****/
//      1ms周期でｶｳﾝﾄｱｯﾌﾟ    
void timmsr(void)
{
    mtm_ms++;                  // 1ms
    if(mtm_ms >= 1000){        // 1sec
        mtm_ms = 0;
        mtm_sec++;
        if(mtm_sec >= 60){      // 1min
            f_mtm1m = 1;        // 1分ﾌﾗｸﾞ
            mtm_sec = 0;
            if(mtm_min < 9999)  mtm_min++;
        }
    }
}

/*****  計測時間クリア      *****/
void mtmclr(void)
{
    mtm_ms  = 0;
    mtm_sec = 0;
    mtm_min = 0;
}

/************************************/
/*      外部D/A出力処理             */
/************************************/

/*****  12bit D/A出力       *****/
/*      LTC1451を制御           */
/*      0～4095                 */
void daout(word d)
{
byte i;
    DACLK_L;                       // CLK L
    DACS_L;                        // CS L
    for(i = 0; i < 12; i++){       // 12bit loop
        if(d & 0x0800){            // bit11をチェック
            DADIN_H;               // data H
        }
        else{
            DADIN_L;               // data L
        }
        DACLK_H;                   // CLK H
        d <<= 1;                   // 左シフト
        DACLK_L;                   // CLK L
    }
    DACS_H;                        // CS H
    DACLK_H;                       // CLK H
    DADIN_H;                       // data L
}

/************************************/
/*      外部A/D入力処理             */
/************************************/

/*****  A/Dﾃﾞｰﾀ      *****/
#define ADCH_SU      5       // A/Dで使うch数 0～4
#define ADAVR_SU    32       // 平均回数
int   ad_data[ADCH_SU];      // A/D生データ
int   ad_avr[ADCH_SU];       // A/D平均値
long  ad_add[ADCH_SU];       // 平均加算値
byte  ad_cnt;                // 平均回数
byte  ad_ch;                 // 変換ch 0～4
boolean f_adok;              // 平均完了ﾌﾗｸﾞ
boolean f_adrst;             // A/Dﾘｽﾀｰﾄﾌﾗｸﾞ

/*****  12bit A/D入力       *****/
//      AD7890を制御
//      chで次変換用のmpxを指定
//      ここでは変換は始めない
//      直前の変換結果を読み出す
//      0 A2 A1 A0 D11 D10～D0の16bit
word adread(byte ch)
{
byte i;
word din,dout;
    ADCS_H;                        // TFS,RFS H
    ADSCLK_H;
    dout = (word)ch << 13;         // A2 A1 A0 CONV STBY～
    ADCS_L;                        // TFS,RFS L
    for(i = 0; i < 16 ; i++){
        if(dout & 0x8000)     ADDIN_H;    // data
        else                  ADDIN_L;
        dout <<= 1;
        ADSCLK_L;
        din  <<= 1;
        if(ADDO_INP)    din |= 0x0001;    // H,L
        ADSCLK_H;
    }
    ADCS_H;                        // TFS,RFS H
    return din;
}

/*****  12bit A/D変換開始指令     *****/
//      AD7890を制御
//      chでmpxを指定(adreadと同じchで)
void adconv(byte ch)
{
byte i;
word dout;
    ADCS_H;                        // TFS,RFS H
    ADSCLK_H;
    dout = ((word)ch << 13) | 0x1000;  // A2 A1 A0 CONV STBY～
    ADCS_L;                        // TFS,RFS L
    for(i = 0; i < 16 ; i++){
        if(dout & 0x8000)     ADDIN_H;    // data
        else                  ADDIN_L;
        dout <<= 1;
        ADSCLK_L;
        ADSCLK_H;
    }
    ADCS_H;                        // TFS,RFS H
}

/*****  A/D 平均処理      *****/
//    1msに1回ｺｰﾙ
//    5chを順次変換
//    指定回数で平均値計算処理
//    平均回数32回なので160msｻｲｸﾙで平均値確定
void adavr(void)
{
byte i, ch;  
int d;
float a;
    if(f_adrst){              // A/D再ｽﾀｰﾄ
        f_adrst = 0;
        f_adok  = 0;
        for(i = 0; i < ADCH_SU; i++){
            ad_add[i] = 0;       // 平均加算値
        }
        ad_cnt = 0;           // 平均回数
        ad_ch  = 0;           // 変換ch 0～4
        adread(0);            // A/D ch0を駆動しておく
        adconv(0);            // ch変換
    }
    else{
//  次変換ﾁｬﾝﾈﾙは一つ後ろ
        ch = ad_ch + 1;
        if(ch == ADCH_SU)    ch = 0;
//  ﾃﾞｰﾀﾘｰﾄﾞとMPX
        d = adread(ch) & 0x0FFF;        // 12bit
        if(d & 0x0800)    d |= 0xF000;  // 符号拡張
//  ﾃﾞｰﾀ保存
        ad_data[ad_ch] = d;
        ad_add[ad_ch]  += (long)d;
        adconv(ch);                     // 新ch変換
//  次ﾃﾞｰﾀ
        ad_ch++;
        if(ad_ch >= ADCH_SU){           // ch一巡
            ad_ch = 0;
            ad_cnt++;
            if(ad_cnt >= ADAVR_SU){     // 平均回数完了 
              for(i = 0; i < ADCH_SU; i++){
                a = (float)ad_add[i] / (float)ADAVR_SU;
                if(a >= 0)    a += 0.5;      // +/-で四捨五入
                else          a -= 0.5;
                ad_avr[i] = (int)a;
                ad_add[i] = 0;
              }
              ad_cnt = 0;
              f_adok = 1;
            }
        }
    }
}

/****************************/
/*    A/D入力スケーリング   */
/****************************/

/*****  A/D入力電圧計算  *****/
//      in  A/D入力ﾃﾞｰﾀ 12bit
//      out x.xxVで返す　+/-の値
int advolt(int d)
{
float a;
    a = (float)d * (float)ADSCL_V; 
    a /= (float)ADSCL_B;
    if(a >= 0)    a += 0.5;      // +/-で四捨五入
    else          a -= 0.5;
    return (int)a;
}

/****************************/
/*    D/A出力スケーリング   */
/****************************/

/****  D/A出力電流計算    *****/
//    in  電流値 x.xxA
//    out D/A設定ﾃﾞｰﾀ 0～4095
word daamp(word d)
{
float a;
    a = (float)d * (float)DASCL_B;
    a /= (float)DASCL_A;
    a += 0.5;                    // 正で四捨五入
    return (word)a;
}

/****************************/
/*    スイッチ入力          */
/****************************/

byte f_swon;       // ｽｲｯﾁ ｵﾝ ﾌﾗｸﾞ

/*****  スイッチ入力    *****/
void swchk(void)
{
static byte n = 0;            // 実行ｽﾃｯﾌﾟ
static byte sw_tm = 0;        // ｽｲｯﾁ ﾁｪｯｸ ﾀｲﾏｰ

//  ｽｲｯﾁ入力ﾁｪｯｸﾀｲﾏｰ
    if(sw_tm)     sw_tm--;
//  ｽｲｯﾁ入力状態ﾁｪｯｸ
    switch(n){
      case 0:    // ｵﾌ確認
        if(SW_INP == 0){            // ｵﾝ ?
          sw_tm = 10;
        }
        else{
          if(sw_tm == 0)    n++;    // 次へ
        } 
        break;
      case 1:    // ｵﾝ待ち
        if(SW_INP == 0){
          sw_tm = 10;
          n++;
        }
        break;
      case 2:    // ｵﾝ安定待ち
        if(SW_INP == 0){            // ｵﾝ継続
          if(sw_tm == 0){
            f_swon = 1;
            n = 0;                  // ｵﾌ待ちへ
          }
        }
        else{                       // ｵﾌになった
            n = 1;                  // ｵﾝ待ちに
        }
        break;
    }
}

/******************************/
/*      定電流ｏｎ／ｏｆｆ    */
/******************************/

boolean  cc_cont;              // CC on/off

/*****  定電流on/off出力  *****/
//    cc_contでｵﾝ,ｵﾌを
void cccont(void)
{
    if(cc_cont)  CC_ON;
    else         CC_OFF;
}

/*****  定電流ｵﾝ      *****/
void ccon(void)
{
    CC_ON;
    cc_cont = 1;
}

/*****  定電流ｵﾌ      *****/
void ccoff(void)
{
    CC_OFF;
    cc_cont = 0;
}

/******************************/
/*      シリアル受信処理      */
/******************************/

#define  RXBF_SIZ    10      // ﾊﾞｯﾌｧ文字数
char rx_bff[RXBF_SIZ+1];     // 受信ﾊﾞｯﾌｧ
byte f_rxok;                 // 受信完了ﾌﾗｸﾞ

/*****  ｼﾘｱﾙ1行受信      *****/
void rxbff(void)
{
static byte cnt = 0;            // 受信文字数  
char c;  
    if(Serial.available()){    // ﾃﾞｰﾀあり
      if(f_rxok == 0){          // 前ﾃﾞｰﾀ受信処理した
        c = Serial.read();
        if(c == '\r'){            // CR?
            rx_bff[cnt] = '\0';   // NULLを最後に
            f_rxok = 1;           // 受信成功
            cnt = 0;              // 最初から
        }
        else{                     // 文字
          if((cnt < RXBF_SIZ) &&  // ﾊﾞｯﾌｧサイズok
             (isprint(c))){       // 20～7E
            rx_bff[cnt] = c;      // ﾊﾞｯﾌｧにｽﾄｱ
            cnt++;                // 1文字進める
          }
        }
      }
    }
}

/*****  xxxxx シリアルデータ出力    *****/
void tx6d(int d)
{
char s[16];
    sprintf(s, "%6d", d);
    Serial.print(s);
}

/*****  xxx.xx シリアルデータ出力    *****/
//    intを1/100してdp付加して出力
void tx4r2h(int d)
{
char s[16];
double a;
    a = (double)d / 100.0;        // 1/100
    dtostrf(a, 7, 2, s);          // ｢%f｣が使えないので
    Serial.print(s);
}

/****************************/
/*        実行              */
/****************************/
//    電流電圧測定データ
word  msr_cur;              // 測定電流
                            // 0.1A～10.0A
word  msrcur_max = 1000;    // max 10.00A
                            // 1で0.01A
word  msr_volt   = 100;     // 放電目標電圧 1.00V
                            // 1で0.01V
word  msr_wait   = 1000;    // 待ち時間 1ms

//    実行区分
byte exc_j = 0;           // 実行区分

#define J_TST      1      // テストモード選択
#define J_TSTAD    3      // A/Dテスト
#define	J_TSTDA    4      // D/Aテスト
#define	J_TSTCUR   6      // CURテスト
#define	J_MSR      8      // 0-10A測定
#define	J_MSR1     9      // 0-10A測定loop
#define J_MSRBK    13     // 中断
#define J_LIF      14     // 放電
#define J_LIFX1    17     // 放電実行

/*****  タイトル出力      *****/
void jttl(void)
{
    Serial.println("# Bat tester (2013-10-21)");
    exc_j++;
}

/*****  テストモード選択      *****/
void jtst(void)
{
    Serial.print("\n# 0:0-10A L:LifeCK ");
    Serial.print("A:A/D D:D/A C:CUR >");
    f_swon = 0;
    f_rxok = 0;
    ccoff();                    // 定電流ｵﾌ
    daout(0);
    exc_j++;
}

/*****  テストモード受信待ち      *****/
void jtstrx(void)
{
char c;
    if(f_rxok){                // 受信あり
        f_rxok = 0;
        f_swon = 0;
        Serial.println();
        c = toupper(rx_bff[0]);    // 先頭文字
        switch(c){
          case '0':
            Serial.println("# 0 -> 10A test");
            Serial.print("# Max Cur(100:1.00A) >");
            exc_j = J_MSR;
            break;
          case 'L':
            Serial.println("# Life test");
            Serial.print("# Cur (100:1.00A) >");
            exc_j = J_LIF;
            break;
          case 'A':
            Serial.println("# A/D test");
            exc_j = J_TSTAD;
            break;
          case 'D':
            Serial.println("# D/A test");
            exc_j = J_TSTDA;
            break;
          case 'C':
            Serial.println("# Cur test");
            exc_j = J_TSTCUR;
            break;
        default:
            Serial.print("# ?");
            exc_j = J_TST;
        }
    }
}

/*****  A/Dテスト    *****/
//    1秒に1行5chぶんを表示
void jtstad(void)
{
byte i;
int d;
    if(tw_1ms == 0){            // ﾀｲﾑｱｯﾌﾟ
        for(i = 0; i < ADCH_SU; i++){      // 5ch
            d = ad_data[i];
            tx6d(d);                       // BCD出力
            tx4r2h(advolt(d));             // xx.xx       
            Serial.print("V");
        }
        Serial.println();
        tw_1ms = 1000;
    }
    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      exc_j = J_TST;
    }
}

/*****  D/Aテスト    *****/
void jtstda(void)
{
    Serial.print("# D/A >");
    exc_j++;
}

/*****  D/Aテスト実行    *****/
void jtstda1(void)
{
word d;
    if(f_swon){              // ｽｲｯﾁでCC on/off
      f_swon = 0;
      cc_cont ^= 1;          // 定電流on/off
      cccont();
    }  
    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      if(!isdigit(rx_bff[0])){   // 先頭文字数字以外
          exc_j = J_TST;
      }
      else{
          d = atoi(rx_bff);
          Serial.println(d, DEC);
          daout(d);
          exc_j--;
      }
    }
}

/*****  CURテスト    *****/
void jtstcur(void)
{
    Serial.print("# CUR >");
    exc_j++;
}

/*****  CURテスト実行    *****/
void jtstcur1(void)
{
word d;
    if(f_swon){              // ｽｲｯﾁでCC on/off
      f_swon = 0;
      cc_cont ^= 1;          // 定電流on/off
      cccont();
    }  
    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      if(!isdigit(rx_bff[0])){   // 先頭文字数字以外
          exc_j = J_TST;
      }
      else{
          d = atoi(rx_bff);
          d = daamp(d);            // AMPからD/A値に
          Serial.print("# D/A=");
          Serial.println(d, DEC);
          daout(d);                // D/A出力
          exc_j--;
      }
    }
}
          
/*****  0-10A測定    *****/
void jmsr(void)
{
word d;
    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      if(!isdigit(rx_bff[0])){   // 先頭文字数字以外
          exc_j = J_TST;
      }
      else{
          msrcur_max = atoi(rx_bff);
          Serial.print("# Start 0.1A -> ");
          tx4r2h(msrcur_max);               // xx.xx
          Serial.println("A");
          Serial.println("#(A)     (V)");
          msr_cur = 0;           // 0.0Aから
          exc_j = J_MSR1;
      }
    }
}

/*****  測定開始   *****/
//      定電流出力
void jmsr1(void)
{
word d;
    daout(daamp(msr_cur));       // 電流出力D/A設定 
    if(msr_cur > 0)     ccon();  // 定電流出力
    tm_1ms = 40;                 // 40ms 安定待ち時間
    tx4r2h(msr_cur);             // xx.xx
    exc_j++;
}

/*****  測定 電流安定待ち   *****/
void jmsr2(void)
{
    if(tm_1ms == 0){        // 安定時間
        f_adrst = 1;        // A/D変換ﾘｽﾀｰﾄ
        f_adok  = 0;
        exc_j++;
    }
}    
  
/*****  測定 A/D変換待ち   *****/
void jmsr3(void)
{
int d,v0,v4;
    if(CCER_INP == 0){      // CC Errあり
        Serial.println("# -CC Err");
        exc_j = J_MSRBK;    // 中断
        return;
    }
    if(f_adok){             // A/D変換完了
        v0 = ad_avr[0];     // 電池1＋
        v4 = ad_avr[4];     // 電池4－
        d = v0 - v4;
        tx4r2h(advolt(d));             // xx.xx
        Serial.println();
//    次電流
        ccoff();                 // 定電流ｵﾌ
        msr_cur += 10;           // 0.1Aｽﾃｯﾌﾟ
        if(msr_cur > msrcur_max){      // 10Amax
            daout(0);
            exc_j = J_TST;
        }
        else{
            tw_1ms = 1000;       // 休止時間
            exc_j++;
        }
    }
}    

/*****  測定 休止   *****/
void jmsr4(void)
{
    if(f_swon){             // ｽｲｯﾁあり?
        Serial.println("# -break");
        exc_j = J_MSRBK;    // 中断
        return;
    }  
    if(tw_1ms == 0){
        exc_j = J_MSR1;      // 繰り返し
    }  
}

/*****  測定 中断    *****/
void jmsrbk(void)
{
    f_swon = 0;
    ccoff();            // 定電流ｵﾌ
    daout(0);
    exc_j = J_TST;      // ﾃｽﾄﾓｰﾄﾞ選択へ
}

/*****    経過時間+電池電圧出力    *****/
void mtmvolt(void)
{
byte i;
int msrv;           // 全体の電圧
int bv[4];          // 4本の電池電圧
word tm;
    tm = mtm_min;                             // 分データ
    if(mtm_sec != 0)   tm++;                  // 00秒以外は+1分
    tx6d(tm);                                 // 分 出力
    msrv = advolt(ad_avr[0] - ad_avr[4]);     // 電池1＋ - 電池4－
    for(i = 0; i < 4; i++){                   // 電池1～電池4
        bv[i] = advolt(ad_avr[i] - ad_avr[i+1]);
    }
    tx4r2h(msrv);                    // xx.XX
    for(i = 0; i < 4; i++){          // 電池1～電池4
        tx4r2h(bv[i]);               // xx.XX
    }
    Serial.println();
}

/*****  放電寿命測定    *****/
//      放電電流入力
void jlif(void)
{
    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      if(!isdigit(rx_bff[0])){   // 先頭文字数字以外
          exc_j = J_TST;
      }
      else{
          msr_cur = atoi(rx_bff);
          tx4r2h(msr_cur);
          Serial.print("A\n# Volt (100:1.00V)>");
          exc_j++;
      }
    }
}

/*****  放電寿命測定    *****/
//      目標電圧入力
void jlif1(void)
{
    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      if(!isdigit(rx_bff[0])){   // 先頭文字数字以外
          exc_j = J_TST;
      }
      else{
          msr_volt = atoi(rx_bff);
          tx4r2h(msr_volt);
          Serial.print("V\n# Wait (ms) >");
          exc_j++;
      }
    }
}

/*****  放電寿命測定    *****/
//      待ち時間入力
void jlif2(void)
{
    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      if(!isdigit(rx_bff[0])){   // 先頭文字数字以外
          exc_j = J_TST;
      }
      else{
          msr_wait = atoi(rx_bff);
          Serial.print(msr_wait, DEC);
          Serial.println("ms");
          mtmclr();           // 計測時間ｸﾘｱ
          f_mtm1m = 1;        // 初回のﾃﾞｰﾀ出力
//        mtmvolt();          // 時間電圧出力
//        f_adrst = 1;        // A/D変換ﾘｽﾀｰﾄ
//        f_adok  = 0;
          exc_j++;
      }
    }
}

/*****  放電寿命測定    *****/
//      定電流出力
void jlifx1(void)
{
    daout(daamp(msr_cur));       // 電流出力D/A設定 
    ccon();                      // 定電流出力
    tm_1ms = 40;                 // 40ms 安定待ち時間
    exc_j++;
}

/*****  放電寿命測定    *****/
//      定電流安定待ち
void jlifx2(void)
{
    if(tm_1ms == 0){        // 安定時間
        f_adrst = 1;        // A/D変換ﾘｽﾀｰﾄ
        f_adok  = 0;
        exc_j++;
    }
}    

/*****  放電寿命測定    *****/
//      A/D変換待ち,データ出力
//      放電終了判定
void jlifx3(void)
{
int d;
    if(CCER_INP == 0){      // CC Errあり
        Serial.println("# -CC Err");
        exc_j = J_MSRBK;    // 中断
        return;
    }
    if(f_mtm1m && f_adok){
        mtmvolt();          // 時間電圧出力
        f_mtm1m = 0;
    }
    if(f_adok){             // A/D変換完了
        d = advolt(ad_avr[0] - ad_avr[4]);     // 電池1＋ - 電池4－
        if(d <= msr_volt){
            mtmvolt();          // 時間電圧出力
            ccoff();            // 定電流ｵﾌ
            daout(0);
            exc_j = J_TST;      // ﾃｽﾄﾓｰﾄﾞ選択へ
        }
        else{
            if(msr_wait != 0){      // 時間待ちあれば
                ccoff();            // 定電流ｵﾌ
            }
            tw_1ms = msr_wait;      // 放電休止時間ｾｯﾄ
            exc_j++;
        }
    }
}

/*****  放電寿命測定    *****/
//      休止時間待ち
void jlifx4(void)
{
    if(f_swon){             // ｽｲｯﾁあり?
        Serial.println("# -Break");
        exc_j = J_MSRBK;    // 中断
        return;
    }  
    if(tw_1ms == 0){
        exc_j = J_LIFX1;        // loop
    }
}

/*****  実行テーブル    *****/
void (*tblexc[])(void)={
        jttl,           //    0  タイトル出力
        jtst,           // *  1  テストモード選択
        jtstrx,         //    2  テストモード受信待ち
	jtstad,		// *  3  A/Dテスト
	jtstda, 	// *  4  D/Aテスト
	jtstda1, 	//    5  D/Aテスト 実行
	jtstcur, 	// *  6  CURテスト
	jtstcur1, 	//    7  CURテスト 実行
	jmsr,		// *  8  0～10A測定
	jmsr1,		// *  9  測定 開始
	jmsr2,		//    10 測定 電流安定待ち
	jmsr3,		//    11 測定 A/D完了待ち
	jmsr4,		//    12 測定 
        jmsrbk,         // *  13 中断
        jlif,           // *  14 放電 電流入力
        jlif1,          //    15 放電 電圧入力
        jlif2,          //    16 放電 待ち時間入力
        jlifx1,         // *  17 放電実行loop
        jlifx2,         //    18 放電実行 定電流安定
        jlifx3,         //    19 放電実行 電圧判定
        jlifx4,         //    20 放電実行 時間待ち
};

/******************************/
/*      セットアップ          */
/******************************/
void setup()
{
//  I/Oイニシャル
    PORTB = 0b00010011;
    DDRB  = 0b11111011;      // port指定
    //           ||||+----  o A/D CS   = H
    //           |||+-----  o A/D SCLK = H
    //           ||+------  i A/D DOUT
    //           |+-------  o A/D CLKIN 1MHz
    //           +--------  o A/D DIN = H
    PORTC = 0b00101111;
    DDRC  = 0b11010111;
    //          |||||+---  o D/A CLK = H
    //          ||||+----  o D/A DIN = H
    //          |||+-----  o D/A CS = H
    //          ||+------  i CC Err inp
    //          |+-------  o CC on
    //          +--------  i SW inp
    DDRD  = 0b11111100;
    //              |+---  RXD
    //              +----  TXD
//  A/D用ﾏｽﾀｰｸﾛｯｸ出力
    TCCR1A = 0b01000000;    // OC1A ﾄｸﾞﾙ
    TCCR1B = 0b00001001;    // 1/1
    OCR1A  = 4-1;           // 2MHz
    TCNT1  = 0;
//  ﾀｲﾏｰ計時
    MsTimer2::set(1, tm2int);   // 1msｻｲｸﾙ割り込み処理
    MsTimer2::start();

//  シリアル通信
    Serial.begin(9600);

//  12bit A/D ch0を駆動しておく
    adread(0);           // ch0 MPX
    adconv(0);           // 変換
}

/******************************/
/*      ループ                */
/******************************/
void loop()
{
    sbi(PIND,PIND7);          // 13pin ﾄｸﾞﾙ
    if(f_tm1ms){
      f_tm1ms = 0;            // 1msごとにｺｰﾙ
      if(tw_1ms)      tw_1ms--;
      swchk();                // ｽｲｯﾁ入力ﾁｪｯｸ
      adavr();                // A/D平均処理
      timmsr();               // 計測時間ﾀｲﾏｰ計時
    }
    rxbff();                  // ｼﾘｱﾙ1行受信
    tblexc[exc_j]();          // テーブル実行
}


/*=====  end of "BAT5A01"    =====*/

#if 0
/*****  A/Dテスト    *****/
void jtstad(void)
{
int  data[8];    // +/-で
word d,ch;
byte i;
char s[32];
    if(tw_1ms == 0){            // ﾀｲﾑｱｯﾌﾟ
        tw_1ms = 1000;
        for(i = 0; i < 5; i++){      // 5ch
            d = adread(i);           // A/D読み出し
            adconv(i);               // 変換
//  前変換ﾃﾞｰﾀに対する処理
            ch = (d >> 12) & 0x0007;    // 0～7
            d &= 0x0FFF;                // 12bit
            if(d & 0x0800)    d |= 0xF000;  // 符号拡張
            data[ch] = d;
        }
        for(i = 0; i < 5; i++){      // 5ch
            sprintf(s, " %5d",  data[i]);
            Serial.print(s);
        }
        Serial.println();
    }

    if(f_rxok){                  // 受信ﾁｪｯｸ
      f_rxok = 0;
      exc_j = J_TSTCK;
    }
}
#endif

